@@ -1,6 +1,6 @@
 # --
 # Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
-# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -17,24 +17,21 @@ use parent qw(Kernel::System::Console::BaseCommand);
 use File::Basename;
 use File::Copy;
 use Lingua::Translit;
-use Pod::Strip;
 
 use Kernel::Language;
-use Kernel::System::VariableCheck qw(DataIsDifferent);
+use Kernel::System::VariableCheck qw(:all);
 
 our @ObjectDependencies = (
     'Kernel::Config',
     'Kernel::System::DateTime',
     'Kernel::System::Encode',
     'Kernel::System::Main',
-    'Kernel::System::Storable',
-    'Kernel::System::SysConfig',
 );
 
 sub Configure {
     my ( $Self, %Param ) = @_;
 
-    $Self->Description('Update the OTRS translation files.');
+    $Self->Description('Update the Znuny translation files.');
     $Self->AddOption(
         Name        => 'language',
         Description => "Which language to use, omit to update all languages.",
@@ -44,7 +41,7 @@ sub Configure {
     );
     $Self->AddOption(
         Name        => 'module-directory',
-        Description => "Translate the OTRS module in the given directory.",
+        Description => "Translate the Znuny module in the given directory.",
         Required    => 0,
         HasValue    => 1,
         ValueRegex  => qr/.*/smx,
@@ -56,12 +53,19 @@ sub Configure {
         Required => 0,
         HasValue => 0,
     );
+    $Self->AddOption(
+        Name => 'keep-old',
+        Description =>
+            "Keep old language files (e.g. Kernel/Language/de_GeneralCatalog.pm). This is only needed if you want to diff these files.",
+        Required => 0,
+        HasValue => 0,
+    );
 
     my $Name = $Self->Name();
 
     $Self->AdditionalHelp(<<"EOF");
 
-<yellow>Translating OTRS</yellow>
+<yellow>Translating Znuny</yellow>
 
 Make sure that you have a clean system with a current configuration. No modules may be installed or linked into the system!
 
@@ -79,16 +83,47 @@ EOF
 
 my $BreakLineAfterChars = 60;
 
+sub PreRun {
+    my ( $Self, %Param ) = @_;
+
+    my $Home = $Kernel::OM->Get('Kernel::Config')->Get('Home');
+
+    $Self->Print("<yellow>Check for symbolic links...</yellow>\n\n");
+
+    my @FilesInDirectory = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
+        Directory => $Home,
+        Filter    => '*',
+        Silent    => 1,
+    );
+
+    my $LinkedFile = 0;
+    for my $File (@FilesInDirectory) {
+        if ( -l "$File" ) {
+            $LinkedFile++;
+            $Self->Print("<red>Linked file detected:</red> $File\n");
+        }
+    }
+
+    return $Self->ExitCodeOk() if !$LinkedFile;
+
+    $Self->Print("\n<red>Make sure that all symbolic links are removed before.</red>\n");
+    $Self->Print("<green>perl module-tools/bin/otrs.ModuleTools.pl Module::File::Unlink --all $Home</green>\n\n");
+
+    die;
+}
+
 sub Run {
     my ( $Self, %Param ) = @_;
 
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+
     my @Languages;
     my $LanguageOption = $Self->GetOption('language');
-    my $Home           = $Kernel::OM->Get('Kernel::Config')->Get('Home');
+    my $Home           = $ConfigObject->Get('Home');
 
     # check params
     if ( !$LanguageOption ) {
-        my %DefaultUsedLanguages = %{ $Kernel::OM->Get('Kernel::Config')->Get('DefaultUsedLanguages') };
+        my %DefaultUsedLanguages = %{ $ConfigObject->Get('DefaultUsedLanguages') };
         @Languages = sort keys %DefaultUsedLanguages;
         @Languages = grep { $_ ne 'en' } @Languages;    # ignore en*.pm files
     }
@@ -141,8 +176,17 @@ my %UsedInJS;
 sub HandleLanguage {
     my ( $Self, %Param ) = @_;
 
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+    my $MainObject   = $Kernel::OM->Get('Kernel::System::Main');
+
     my $Language = $Param{Language};
-    my $Module   = $Param{Module} || '';
+
+    # Language file, which contains all translations
+    my $LanguageObject = Kernel::Language->new(
+        UserLanguage => $Language,
+    );
+
+    my $Module = $Param{Module} || '';
 
     my $ModuleDirectory = $Module;
     my $LanguageFile;
@@ -151,7 +195,7 @@ sub HandleLanguage {
     my $TargetPOFile;
     my $IsSubTranslation;
 
-    my $DefaultTheme = $Kernel::OM->Get('Kernel::Config')->Get('DefaultTheme');
+    my $DefaultTheme = $ConfigObject->Get('DefaultTheme');
 
     # We need to map internal codes to the official ones used by Weblate
     my %WeblateLanguagesMap = (
@@ -160,13 +204,13 @@ sub HandleLanguage {
     );
 
     my $WeblateLanguage = $WeblateLanguagesMap{$Language} // $Language;
-    my $Home            = $Kernel::OM->Get('Kernel::Config')->Get('Home');
+    my $Home            = $ConfigObject->Get('Home');
 
     if ( !$Module ) {
         $LanguageFile  = "$Home/Kernel/Language/$Language.pm";
         $TargetFile    = "$Home/Kernel/Language/$Language.pm";
-        $TargetPOTFile = "$Home/i18n/otrs/otrs.pot";
-        $TargetPOFile  = "$Home/i18n/otrs/otrs.$WeblateLanguage.po";
+        $TargetPOTFile = "$Home/i18n/Znuny/Znuny.pot";
+        $TargetPOFile  = "$Home/i18n/Znuny/Znuny.$WeblateLanguage.po";
     }
     else {
         $IsSubTranslation = 1;
@@ -201,375 +245,43 @@ sub HandleLanguage {
 
     if ( !@OriginalTranslationStrings ) {
 
-        $Self->Print(
-            "Extracting source strings, this can take a moment...\n"
+        $Self->Print("<yellow>Extracting source strings, this can take a moment.</yellow>\n");
+
+        # open .tt files and save all strings into @OriginalTranslationStrings
+        $Self->Print("<yellow>Extracting TT Templates...</yellow>");
+        push @OriginalTranslationStrings, $LanguageObject->GetTTTemplateTranslatableStrings(
+            ModuleDirectory => $ModuleDirectory,
         );
+        $Self->Print("<green> Done.</green>\n");
+
+        $Self->Print("<yellow>Extracting JS Templates...</yellow>");
+        push @OriginalTranslationStrings, $LanguageObject->GetJSTemplateTranslatableStrings(
+            ModuleDirectory => $ModuleDirectory,
+        );
+        $Self->Print("<green> Done.</green>\n");
+
+        $Self->Print("<yellow>Extracting PerlModule...</yellow>");
+        push @OriginalTranslationStrings, $LanguageObject->GetPerlModuleTranslatableStrings(
+            ModuleDirectory => $ModuleDirectory,
+        );
+        $Self->Print("<green> Done.</green>\n");
+
+        $Self->Print("<yellow>Extracting XML...</yellow>");
+        push @OriginalTranslationStrings, $LanguageObject->GetXMLTranslatableStrings(
+            ModuleDirectory => $ModuleDirectory,
+        );
+        $Self->Print("<green> Done.</green>\n");
+
+        $Self->Print("<yellow>Extracting JS...</yellow>");
+        push @OriginalTranslationStrings, $LanguageObject->GetJSTranslatableStrings(
+            ModuleDirectory => $ModuleDirectory,
+        );
+        $Self->Print("<green> Done.</green>\n");
+
+        $Self->Print("<yellow>Extracting SysConfig...</yellow>");
+        push @OriginalTranslationStrings, $LanguageObject->GetSysConfigTranslatableStrings();
+        $Self->Print("<green> Done.</green>\n\n");
 
-        # open .tt files and write new translation file
-        my %UsedWords;
-        my $TemplatesDirectory = $IsSubTranslation
-            ? "$ModuleDirectory/Kernel/Output/HTML/Templates/$DefaultTheme"
-            : "$Home/Kernel/Output/HTML/Templates/$DefaultTheme";
-
-        my @TemplateList;
-        if ( -d $TemplatesDirectory ) {
-            @TemplateList = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                Directory => $TemplatesDirectory,
-                Filter    => '*.tt',
-                Recursive => 1,
-            );
-        }
-
-        my $CustomTemplatesDir = "$ModuleDirectory/Custom/Kernel/Output/HTML/Templates/$DefaultTheme";
-        if ( $IsSubTranslation && -d $CustomTemplatesDir ) {
-            my @CustomTemplateList = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                Directory => $CustomTemplatesDir,
-                Filter    => '*.tt',
-                Recursive => 1,
-            );
-            push @TemplateList, @CustomTemplateList;
-        }
-
-        for my $File (@TemplateList) {
-
-            my $ContentRef = $Kernel::OM->Get('Kernel::System::Main')->FileRead(
-                Location => $File,
-                Mode     => 'utf8',
-            );
-
-            if ( !ref $ContentRef ) {
-                die "Can't open $File: $!";
-            }
-
-            my $Content = ${$ContentRef};
-
-            $File =~ s{^.*/(.+?)\.tt}{$1}smx;
-
-            # do translation
-            $Content =~ s{
-                Translate\(
-                    \s*
-                    (["'])(.*?)(?<!\\)\1
-            }
-            {
-                my $Word = $2 // '';
-
-                # unescape any \" or \' signs
-                $Word =~ s{\\"}{"}smxg;
-                $Word =~ s{\\'}{'}smxg;
-
-                if ( $Word && !$UsedWords{$Word}++ ) {
-
-                    push @OriginalTranslationStrings, {
-                        Location => "Template: $File",
-                        Source => $Word,
-                    };
-
-                }
-
-                '';
-            }egx;
-        }
-
-        # Add strings from .html.tmpl files (JavaScript templates).
-        my $JSTemplatesDirectory = $IsSubTranslation
-            ? "$ModuleDirectory/Kernel/Output/JavaScript/Templates/$DefaultTheme"
-            : "$Home/Kernel/Output/JavaScript/Templates/$DefaultTheme";
-
-        my @JSTemplateList;
-        if ( -d $JSTemplatesDirectory ) {
-            @JSTemplateList = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                Directory => $JSTemplatesDirectory,
-                Filter    => '*.html.tmpl',
-                Recursive => 1,
-            );
-        }
-
-        my $CustomJSTemplatesDir = "$ModuleDirectory/Custom/Kernel/Output/JavaScript/Templates/$DefaultTheme";
-        if ( $IsSubTranslation && -d $CustomJSTemplatesDir ) {
-            my @CustomJSTemplateList = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                Directory => $CustomJSTemplatesDir,
-                Filter    => '*.html.tmpl',
-                Recursive => 1,
-            );
-            push @JSTemplateList, @CustomJSTemplateList;
-        }
-
-        for my $File (@JSTemplateList) {
-
-            my $ContentRef = $Kernel::OM->Get('Kernel::System::Main')->FileRead(
-                Location => $File,
-                Mode     => 'utf8',
-            );
-
-            if ( !ref $ContentRef ) {
-                die "Can't open $File: $!";
-            }
-
-            my $Content = ${$ContentRef};
-
-            $File =~ s{^.*/(.+?)\.html\.tmpl}{$1}smx;
-
-            # Find strings marked for translation.
-            $Content =~ s{
-                \{\{
-                \s*
-                (["'])(.*?)(?<!\\)\1
-                \s*
-                \|
-                \s*
-                Translate
-            }
-            {
-                my $Word = $2 // '';
-
-                # Unescape any \" or \' signs.
-                $Word =~ s{\\"}{"}smxg;
-                $Word =~ s{\\'}{'}smxg;
-
-                if ( $Word && !$UsedWords{$Word}++ ) {
-                    push @OriginalTranslationStrings, {
-                        Location => "JS Template: $File",
-                        Source   => $Word,
-                    };
-                }
-
-                # Also save that this string was used in JS (for later use in Loader).
-                $UsedInJS{$Word} = 1;
-
-                '';
-            }egx;
-        }
-
-        # add translatable strings from Perl code
-        my $PerlModuleDirectory = $IsSubTranslation
-            ? "$ModuleDirectory/Kernel"
-            : "$Home/Kernel";
-
-        my @PerlModuleList;
-        if ( -d $PerlModuleDirectory ) {
-            @PerlModuleList = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                Directory => $PerlModuleDirectory,
-                Filter    => '*.pm',
-                Recursive => 1,
-            );
-        }
-
-        # include Custom folder for modules
-        my $CustomKernelDir = "$ModuleDirectory/Custom/Kernel";
-        if ( $IsSubTranslation && -d $CustomKernelDir ) {
-            my @CustomPerlModuleList = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                Directory => $CustomKernelDir,
-                Filter    => '*.pm',
-                Recursive => 1,
-            );
-            push @PerlModuleList, @CustomPerlModuleList;
-        }
-
-        # Include some additional folders for modules.
-        for my $AdditionalFolder ( 'var/packagesetup', 'var/processes/examples', 'var/webservices/examples' ) {
-            if ( $IsSubTranslation && -d "$ModuleDirectory/$AdditionalFolder" ) {
-                my @PackageSetupModuleList = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                    Directory => "$ModuleDirectory/$AdditionalFolder",
-                    Filter    => '*.pm',
-                    Recursive => 1,
-                );
-                push @PerlModuleList, @PackageSetupModuleList;
-            }
-        }
-
-        FILE:
-        for my $File (@PerlModuleList) {
-
-            next FILE if ( $File =~ m{cpan-lib}xms );
-            next FILE if ( $File =~ m{Kernel/Config/Files}xms );
-
-            my $ContentRef = $Kernel::OM->Get('Kernel::System::Main')->FileRead(
-                Location => $File,
-                Mode     => 'utf8',
-            );
-
-            if ( !ref $ContentRef ) {
-                die "Can't open $File: $!";
-            }
-
-            $File =~ s{^.*/(Kernel/)}{$1}smx;
-            $File =~ s{^.*/(var/)}{$1}smx;
-
-            my $Content = ${$ContentRef};
-
-            # Remove POD
-            my $PodStrip = Pod::Strip->new();
-            $PodStrip->replace_with_comments(1);
-            my $Code;
-            $PodStrip->output_string( \$Code );
-            $PodStrip->parse_string_document($Content);
-
-            # Purge all comments
-            $Code =~ s{^ \s* # .*? \n}{\n}xmsg;
-
-            # do translation
-            $Code =~ s{
-                (?:
-                    ->Translate | Translatable
-                )
-                \(
-                    \s*
-                    (["'])(.*?)(?<!\\)\1
-            }
-            {
-                my $Word = $2 // '';
-
-                # unescape any \" or \' signs
-                $Word =~ s{\\"}{"}smxg;
-                $Word =~ s{\\'}{'}smxg;
-
-                # Ignore strings containing variables
-                my $SkipWord;
-                $SkipWord = 1 if $Word =~ m{\$}xms;
-
-                if ( $Word && !$SkipWord && !$UsedWords{$Word}++ ) {
-
-                    push @OriginalTranslationStrings, {
-                        Location => "Perl Module: $File",
-                        Source => $Word,
-                    };
-
-                }
-                '';
-            }egx;
-        }
-
-        # add translatable strings from DB XML
-        my @DBXMLFiles = "$Home/scripts/database/otrs-initial_insert.xml";
-        if ($IsSubTranslation) {
-            @DBXMLFiles = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                Directory => "$ModuleDirectory",
-                Filter    => '*.sopm',
-            );
-        }
-
-        FILE:
-        for my $File (@DBXMLFiles) {
-
-            my $ContentRef = $Kernel::OM->Get('Kernel::System::Main')->FileRead(
-                Location => $File,
-                Mode     => 'utf8',
-            );
-
-            if ( !ref $ContentRef ) {
-                die "Can't open $File: $!";
-            }
-
-            $File =~ s{^.*/(scripts/)}{$1}smx;
-            if ($IsSubTranslation) {
-                $File =~ s{^.*/(.+\.sopm)}{$1}smx;
-            }
-
-            my $Content = ${$ContentRef};
-
-            # do translation
-            $Content =~ s{
-                <(Data|Description)[^>]+Translatable="1"[^>]*>(.*?)</\1>
-            }
-            {
-                my $Word = $2 // '';
-                if ( $Word && !$UsedWords{$Word}++ ) {
-                    push @OriginalTranslationStrings, {
-                        Location => "Database XML / SOPM Definition: $File",
-                        Source => $Word,
-                    };
-                }
-                '';
-            }egx;
-        }
-
-        # add translatable strings from JavaScript code
-        my $JSDirectory = $IsSubTranslation
-            ? "$ModuleDirectory/var/httpd/htdocs/js"
-            : "$Home/var/httpd/htdocs/js";
-
-        my @JSFileList;
-        if ( -d $JSDirectory ) {
-            @JSFileList = $Kernel::OM->Get('Kernel::System::Main')->DirectoryRead(
-                Directory => $JSDirectory,
-                Filter    => '*.js',
-                Recursive => 1,
-            );
-        }
-
-        FILE:
-        for my $File (@JSFileList) {
-
-            my $ContentRef = $Kernel::OM->Get('Kernel::System::Main')->FileRead(
-                Location => $File,
-                Mode     => 'utf8',
-            );
-
-            if ( !ref $ContentRef ) {
-                die "Can't open $File: $!";
-            }
-
-            # skip js cache files
-            next FILE if ( $File =~ m{\/js\/js-cache\/}xmsg );
-
-            my $Content = ${$ContentRef};
-
-            # skip thirdparty files without custom markers
-            if ( $File =~ m{\/js\/thirdparty\/}xmsg ) {
-                next FILE if ( $Content !~ m{\/\/\s*OTRS}xmsg );
-            }
-
-            $File =~ s{^.*/(.+?)\.js}{$1}smx;
-
-            # Purge all comments
-            $Content =~ s{^ \s* // .*? \n}{\n}xmsg;
-
-            # do translation
-            $Content =~ s{
-                (?:
-                    Core.Language.Translate
-                )
-                \(
-                    \s*
-                    (["'])(.*?)(?<!\\)\1
-            }
-            {
-                my $Word = $2 // '';
-
-                # unescape any \" or \' signs
-                $Word =~ s{\\"}{"}smxg;
-                $Word =~ s{\\'}{'}smxg;
-
-                if ( $Word && !$UsedWords{$Word}++ ) {
-
-                    push @OriginalTranslationStrings, {
-                        Location => "JS File: $File",
-                        Source => $Word,
-                    };
-
-                }
-
-                # also save that this string was used in JS (for later use in Loader)
-                $UsedInJS{$Word} = 1;
-
-                '';
-            }egx;
-        }
-
-        # add translatable strings from SysConfig
-        my @Strings = $Kernel::OM->Get('Kernel::System::SysConfig')->ConfigurationTranslatableStrings();
-
-        STRING:
-        for my $String ( sort @Strings ) {
-
-            next STRING if !$String || $UsedWords{$String}++;
-
-            push @OriginalTranslationStrings, {
-                Location => 'SysConfig',
-                Source   => $String,
-            };
-        }
     }
 
     if ($IsSubTranslation) {
@@ -583,17 +295,12 @@ sub HandleLanguage {
         );
     }
 
-    # Language file, which only contains the OTRS core translations
+    # Language file, which only contains the Znuny core translations
     my $LanguageCoreObject = Kernel::Language->new(
         UserLanguage    => $Language,
         TranslationFile => 1,
     );
 
-    # Language file, which contains all translations
-    my $LanguageObject = Kernel::Language->new(
-        UserLanguage => $Language,
-    );
-
     # Helpers for SR Cyr2Lat Transliteration
     my $TranslitObject;
     my $TranslitLanguageCoreObject;
@@ -677,6 +384,10 @@ sub HandleLanguage {
         );
     }
 
+    if ( !%UsedInJS && IsHashRefWithData( $LanguageObject->{UsedInJS} ) ) {
+        %UsedInJS = %{ $LanguageObject->{UsedInJS} };
+    }
+
     $Self->WritePerlLanguageFile(
         IsSubTranslation   => $IsSubTranslation,
         LanguageCoreObject => $LanguageCoreObject,
@@ -685,9 +396,8 @@ sub HandleLanguage {
         LanguageFile       => $LanguageFile,
         TargetFile         => $TargetFile,
         TranslationStrings => \@TranslationStrings,
-        UsedInJS           => \%UsedInJS,
+        UsedInJS           => \%UsedInJS,             # Remember which strings came from JavaScript
     );
-
     return 1;
 }
 
@@ -696,7 +406,9 @@ sub LoadPOFile {
 
     return if !-e $Param{TargetPOFile};
 
-    $Kernel::OM->Get('Kernel::System::Main')->Require('Locale::PO') || die "Could not load Locale::PO";
+    my $MainObject = $Kernel::OM->Get('Kernel::System::Main');
+
+    $MainObject->Require('Locale::PO') || die "Could not load Locale::PO";
     my $POEntries = Locale::PO->load_file_asarray( $Param{TargetPOFile} );
 
     my %POTranslations;
@@ -724,7 +436,10 @@ sub LoadPOFile {
 sub WritePOFile {
     my ( $Self, %Param ) = @_;
 
-    $Kernel::OM->Get('Kernel::System::Main')->Require('Locale::PO') || die "Could not load Locale::PO";
+    my $MainObject   = $Kernel::OM->Get('Kernel::System::Main');
+    my $EncodeObject = $Kernel::OM->Get('Kernel::System::Encode');
+
+    $MainObject->Require('Locale::PO') || die "Could not load Locale::PO";
 
     if ( !-e $Param{TargetPOFile} ) {
         File::Copy::copy( $Param{TargetPOTFile}, $Param{TargetPOFile} )
@@ -737,7 +452,7 @@ sub WritePOFile {
     for my $Entry ( @{$POEntries} ) {
         my $Source = $Entry->dequote( $Entry->msgid() );
         $Source =~ s/\\{2}/\\/g;
-        $Kernel::OM->Get('Kernel::System::Encode')->EncodeInput( \$Source );
+        $EncodeObject->EncodeInput( \$Source );
         $POLookup{$Source} = $Entry;
     }
 
@@ -745,10 +460,10 @@ sub WritePOFile {
 
         my $Source = $String->{Source};
         $Source =~ s/\\/\\\\/g;
-        $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput( \$Source );
+        $EncodeObject->EncodeOutput( \$Source );
         my $Translation = $String->{Translation};
         $Translation =~ s/\\/\\\\/g;
-        $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput( \$Translation );
+        $EncodeObject->EncodeOutput( \$Translation );
 
         # Is there an entry in the PO already?
         if ( exists $POLookup{ $String->{Source} } ) {
@@ -781,9 +496,11 @@ sub WritePOTFile {
 
     my @POTEntries;
 
-    $Kernel::OM->Get('Kernel::System::Main')->Require('Locale::PO') || die "Could not load Locale::PO";
+    my $MainObject = $Kernel::OM->Get('Kernel::System::Main');
 
-    my $Package = $Param{Module} // 'OTRS';
+    $MainObject->Require('Locale::PO') || die "Could not load Locale::PO";
+
+    my $Package = $Param{Module} // 'Znuny';
 
     # build creation date, only YEAR-MO-DA HO:MI is needed without seconds
     my $CreationDate = $Kernel::OM->Create('Kernel::System::DateTime')->Format(
@@ -843,6 +560,8 @@ sub WritePOTFile {
 sub WritePerlLanguageFile {
     my ( $Self, %Param ) = @_;
 
+    my $MainObject = $Kernel::OM->Get('Kernel::System::Main');
+
     my $LanguageCoreObject = $Param{LanguageCoreObject};
 
     my $Indent = ' ' x 8;    # 8 spaces for core files
@@ -924,7 +643,7 @@ sub WritePerlLanguageFile {
 
         $NewOut = <<"EOF";
 $Separator
-# Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
+# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
 $Separator
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -964,8 +683,8 @@ EOF
 
                 $NewOut .= "    # possible charsets\n";
                 $NewOut .= "    \$Self->{Charset} = [";
-                for ( $LanguageCoreObject->GetPossibleCharsets() ) {
-                    $NewOut .= "'$_', ";
+                for my $Charset ( $LanguageCoreObject->GetPossibleCharsets() ) {
+                    $NewOut .= "'$Charset', ";
                 }
                 $NewOut .= "];\n";
                 my $Completeness = 0;
@@ -1014,11 +733,14 @@ EOF
 
     my $TargetFile = $Param{TargetFile};
 
-    if ( -e $TargetFile ) {
+    if ( -e $TargetFile && $Self->GetOption('keep-old') ) {
         rename( $TargetFile, "$TargetFile.old" ) || die $!;
     }
+    elsif ( -e $TargetFile ) {
+        unlink($TargetFile);
+    }
 
-    $Kernel::OM->Get('Kernel::System::Main')->FileWrite(
+    $MainObject->FileWrite(
         Location => $TargetFile,
         Content  => \$NewOut,
         Mode     => 'utf8',        # binmode|utf8
